# Backend for Melting Enthalpy and Temperature predictor implementation
#
# Arun S. Moorthy, amoorthy@uoguelph.ca
# University of Guelph
#
# Date: 2016-07-04
#--------------------------------------------------------------------------------------
# ARUN DEFINED FUNCTIONS---------------------------------------------------------------
#--------------------------------------------------------------------------------------
MolWeightCalc <- function(nc,nd){   #Seems good
  mwC = 12.0107;
  mwH = 1.0079;
  mwO = 15.9994;
  
  head = 3*mwC + 5*mwH;
  esterLink = 6*mwO + 3*mwC;
  body = (nc-6)*(mwC+2*mwH) - 2*mwH*nd;
  end = 3*(mwC+3*mwH);
  MW = head + esterLink + body + end;
  return(MW);
}

fodd <- function(a,b){
  if (a%%b==0){x=0;}
  else{x=1;}
  return(x);
}

# Question: when x and y = 0, why does fxy not equal 0?
fxy <- function(x,xo,kx,y,ky){
  z = 2 - exp(-1*((x-xo)/kx)^2) - exp(-(y/ky)^2)
  return(z);
}

EnthalpyFusionUnsat <- function(hfsat,hol,nol,hel,nel, hli,nli){
  Hf = hfsat + hol*nol + hel*nel + hli*nli;
  return(Hf);
}

EnthalpyFusionSat <- function(nc,h,ho,hxy,fxy,hodd,fodd){
  Hf = h*nc +ho +hxy*fxy +hodd*fodd;
  return(Hf)
}

EntropyFusionSat <- function(nc,s,so,sxy,fxy,sodd,fodd,fasym){
  Ef = s*nc +so +sxy*fxy +sodd*fodd +8.314*log(2)*fasym;
  return(Ef)
}

MeltingTemp <- function(nc,Tinf,A,B){
  Tf = Tinf*(1+ A/nc - (A*B)/(nc^2));
  return(Tf)
}

SatA <-function(x,y,Ao,Aodd,fodd,Ax,Ax2,Axy,Ay,Ay2){
  A = Ao + Aodd*fodd + Ax*x + Ax2*x^2 + Axy*x*y + Ay*y + Ay2*y^2;
  return(A);
}
#--------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------
# CODE START
#--------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------
if(polymorph=="alpha"){
  parCol = 2;  
} else if (polymorph=="beta prime"){
  parCol = 3;
} else {
  parCol = 4;
}

nc1 = FattyAcids[FattyAcids$Symbol==s1,3];
nc2 = FattyAcids[FattyAcids$Symbol==s2,3];
nc3 = FattyAcids[FattyAcids$Symbol==s3,3];

nd1 = FattyAcids[FattyAcids$Symbol==s1,4];
nd2 = FattyAcids[FattyAcids$Symbol==s2,4];
nd3 = FattyAcids[FattyAcids$Symbol==s3,4];

ncT = nc1+nc2+nc3;
ndT = nd1+nd2+nd3;

p = min(nc1,nc3);
q = nc2;
r = max(nc1,nc3);

x = q-p;
X = x;
#if(x>6){x=6;} # is this a real rule?

y = r-p;
Y = y;
#if(y>6){y=6;} # ditto

# Re-copy databases for ease of use
Phf = HEstimateParameters;
Pt = TEstimateParameters;
UPhf = HEstimateParametersUNSAT;
UPt = TEstimateParametersUNSAT;

TAG = paste(s1,s2,s3,sep="")
iTAG = paste(s3,s2,s1,sep="")
mwSample = MolWeightCalc(ncT,ndT);

oddness = fodd(nc1,2) + fodd(nc2,2) + fodd(nc3,2);
if(oddness>0){fodd=1}else{fodd=0}

fxy = fxy(x,Phf[8,parCol],Phf[7,parCol],y,Phf[7,parCol]);

if(y==0 && s1==s3){fasym=0}else{fasym=1};
if(polymorph=="beta"){isBeta=1}else{isBeta=0};
if(isBeta==1){hodd=Phf[10,parCol]}else{hodd=0};

hfSample = EnthalpyFusionSat(ncT,Phf[2,parCol],Phf[1,parCol],Phf[5,parCol],fxy,hodd,fodd);
hfSatSample = hfSample;
efSample = EntropyFusionSat(ncT,Phf[4,parCol],Phf[3,parCol],Phf[6,parCol],fxy,hodd,fodd,fasym*isBeta);
efSatSample = efSample;

# Enthalpy For Unsaturated Samples:
nol = 0;
nel = 0;
nli = 0;
nlin = 0;

if (ndT>=1){
  if(s1=="O"){nol=nol+1;} else if (s1=="D"){nel = nel+1;} else if (s1=="J"){nli = nli+1;} else if (s1=="N"){nlin = nlin+1;}
  if(s2=="O"){nol=nol+1;} else if (s2=="D"){nel = nel+1;} else if (s2=="J"){nli = nli+1;} else if (s2=="N"){nlin = nlin+1;}
  if(s3=="O"){nol=nol+1;} else if (s3=="D"){nel = nel+1;} else if (s3=="J"){nli = nli+1;} else if (s3=="N"){nlin = nlin+1;}
  
  hfSample = EnthalpyFusionUnsat(hfSatSample,UPhf[1,parCol],nol, UPhf[2,parCol],nel, UPhf[3,parCol],nli);
}

nOO = max(0,nol-1);
nDD = 0; #max(0,nel-1);
nJJ = max(0,nli-1);
nNN = max(0,nlin-1);

nOJ = nol*nli;
nON = nol*nlin;
nJN = nli*nlin;

# Melting Temperature
Tinfa = Phf[9,parCol];
Tinfb = Pt[15,parCol];

#B0 = Phf[3,parCol];
#A0 = Phf[1,parCol];
B0 = Phf[3,parCol] + Phf[6,parCol]*fxy + 8.314*log(2)*fasym*isBeta; # so'
A0 = Phf[1,parCol] + Phf[5,parCol]*fxy; # ho'

A1a = A0/Phf[2,parCol] - B0/Phf[4,parCol];
B1a = B0/Phf[4,parCol]

A1b = SatA(x,y,Pt[1,parCol],Pt[2,parCol],fodd,Pt[3,parCol],Pt[4,parCol],Pt[5,parCol],Pt[6,parCol],Pt[7,parCol]);
B1b = SatA(x,y,Pt[8,parCol],Pt[9,parCol],fodd,Pt[10,parCol],Pt[11,parCol],Pt[12,parCol],Pt[13,parCol],Pt[14,parCol]);

aO = UPt[1,parCol];
aD = UPt[2,parCol];
aJ = UPt[3,parCol];
aN = UPt[4,parCol];
aOO = UPt[5,parCol];
aDD = UPt[6,parCol];
aJJ = UPt[7,parCol];
aNN = UPt[8,parCol];
aOJ = UPt[9,parCol];
aON = UPt[10,parCol];
aJN = UPt[11,parCol];

Aa = (A1a + aO*nol + aD*nel + aJ*nli + aN*nlin + 
          aOO*nOO + aDD*nDD + aJJ*nJJ + aNN*nNN+
          aOJ*nOJ + aON*nON + aJN*nJN);
Ab = (A1b + aO*nol + aD*nel + aJ*nli + aN*nlin + 
        aOO*nOO + aDD*nDD + aJJ*nJJ + aNN*nNN+
        aOJ*nOJ + aON*nON + aJN*nJN);


bO = UPt[12,parCol];
bJ = UPt[13,parCol];
bN = UPt[14,parCol];

Ba = B1a + bO*nol + bN*nlin + bJ*nli;
Bb = B1b + bO*nol + bN*nlin + bJ*nli;

# if(ndT>0){
  tfSamplea = MeltingTemp(ncT,Tinfa,Aa,Ba) - 273.16;  
  tfSampleb = MeltingTemp(ncT,Tinfb,Ab,Bb) - 273.16;

# } else {
#   tfSample = hfSample*1000/efSample-273.16;  
# }



## Output Information
nCarbons = ncT+3;
nOxygens = 6;
nHydrogens = 2*(ncT-6)+14-2*ndT;

ChemForm = paste("C",nCarbons,"-O",nOxygens,"-H",nHydrogens,sep="");

# SummaryOutput = array(0,dim=c(8,2));
# SummaryOutput[1,1] = "Reference ID";

ExpInfo = array(0,dim=c(1,4));

a = which(ExperResults[,1]==TAG);

if (length(a)==0){
  ExpInfo <- c(NA,NA,NA,NA)
  
  a2 = which(ExperResults[,1]==iTAG);
  
  if (length(a2)==0){
    ExpInfo <-c(NA,NA,NA,NA);
  } else {
    
    if(polymorph=="alpha"){
      indexi=2;
    } else if (polymorph=="beta prime"){
      indexi=4;
    } else {
      indexi=6;
    }
    
    ExpInfo[1] = ExperResults[ExperResults$TAG==iTAG,indexi];
    ExpInfo[2] = ExperResults[ExperResults$TAG==iTAG,indexi+1];
    ExpInfo[3] = ExperResults[ExperResults$TAG==iTAG,indexi+6];
    ExpInfo[4] = ExperResults[ExperResults$TAG==iTAG,indexi+7];
  }

  } else {

  if(polymorph=="alpha"){
    indexi=2;
  } else if (polymorph=="beta prime"){
    indexi=4;
  } else {
    indexi=6;
  }
  
  ExpInfo[1] = ExperResults[ExperResults$TAG==TAG,indexi];
  ExpInfo[2] = ExperResults[ExperResults$TAG==TAG,indexi+1];
  ExpInfo[3] = ExperResults[ExperResults$TAG==TAG,indexi+6];
  ExpInfo[4] = ExperResults[ExperResults$TAG==TAG,indexi+7];
}




if(shouldDisp==1){
# Display
cat("TAG: ")
cat(paste(polymorph,TAG));
cat("\n");
cat("Chem. Formula: ")
cat(ChemForm);
cat("\n");
cat("X: ")
cat(X);
cat("\n")
cat("Y: ")
cat(Y);
cat("\n")
cat("nc: ")
cat(ncT);
cat("\n")
cat("u: ")
cat(ndT);
cat("\n")
cat("MW: ")
cat(mwSample);
cat("\n")
cat("Hf: ")
cat(hfSample);
cat("\n")
cat("Hf experiment: ")
cat(paste(ExpInfo[1], " from ", ExpInfo[2], " experiments"));
cat("\n")
cat("Tf -a : ")
cat(tfSamplea);
cat("\n")
cat("Tf -b : ")
cat(tfSampleb);
cat("\n")
cat("Tf experiment: ")
cat(paste(ExpInfo[3], " from ", ExpInfo[4], " experiments"));
cat("\n")
}

Results4ShinyApp = as.data.frame(array(" ", dim=c(5,3)),stringsAsFactors=FALSE);
row.names(Results4ShinyApp) <-c("Reference ID:",
                                "Molecular Formula:", 
                                "Molar Mass [g/mol]:",
                                "Enthalpy [kJ/mol]:",
                                "Temperature [deg. C]:");


names(Results4ShinyApp)<-c(" "," "," "); # Column Names
Results4ShinyApp[1,1] = as.character(paste(polymorph,TAG));
Results4ShinyApp[2,1] = as.character(ChemForm);
Results4ShinyApp[3,1] = as.character(round(mwSample,1));
Results4ShinyApp[4,1] = as.character(ExpInfo[1]);
Results4ShinyApp[5,1] = as.character(ExpInfo[3]);
# Results4ShinyApp[8,2] = "deg. Celsius";

Results4ShinyApp2 = as.data.frame(array(" ",dim=c(2,3)),stringsAsFactors=FALSE);
row.names(Results4ShinyApp2) <-c (
          "Predicted Enthalpy [kJ/mol]",
          "Predicted Temperature [deg. Celsius]");

names(Results4ShinyApp2)<-c("Wesdorp-Moorthy [a]","Wesdorp-Moorthy [b]","GC Method")
Results4ShinyApp2[1,1] = as.character(round(hfSample,1));
Results4ShinyApp2[2,1] = as.character(round(tfSamplea,1));
Results4ShinyApp2[2,2] = as.character(round(tfSampleb,1));


# Group Contribution Method Addendum

if (s1==s2 && s1==s3 && s2==s3){Class = 1;}
if (s1==s2 && s1!=s3 && s2!=s3){Class = 2;}
if (s1!=s2 && s1!=s3 && s2==s3){Class = 2;}
if (s1!=s2 && s1==s3 && s2!=s3){Class = 3;}
if (s1!=s2 && s1!=s3 && s2!=s3){Class = 4;}


acids = colnames(HIJbeta);

y1 = which(acids == s1);
x1 = which(acids == s2);
y2 = which(acids == s3);

if (length(y1)!=0 && length(y2)!=0 && length(x1)!=0){
     
        if (polymorph=="beta"){
          contributionMatrix = HIJbeta;
          contributionMatrixT = TIJbeta;
          Kcol = 4;
        } else if (polymorph == "betaP"){
          contributionMatrix = HIJbprime;
          contributionMatrixT = TIJbprime;
          Kcol = 3;
        } else {
          contributionMatrix = HIJalpha;
          contributionMatrixT = TIJalpha
          Kcol = 2;
        }
      
      cont1 = contributionMatrix[x1,y1];
      cont2 = contributionMatrix[x1,y2];
      
      hfGCM = KH[Class,Kcol] * (cont1 + cont2);
      
      Tcont1 = contributionMatrixT[x1,y1];
      Tcont2 = contributionMatrixT[x1,y2];
      
      tfGCM = KT[Class,Kcol] * (Tcont1 + Tcont2) - 273.15;
      
      
      
} else {
      hfGCM = NA;
      tfGCM = NA;
}

Results4ShinyApp2[1,3] = as.character(round(hfGCM,1));
Results4ShinyApp2[2,3] = as.character(round(tfGCM,1));



# Results4ShinyApp[5,2] = "kJ/mol";
# Results4ShinyApp[6,1] = as.character(round(tfSamplea,1)); 
# Results4ShinyApp[6,2] = "deg. Celsius";
# Results4ShinyApp[7,1] = as.character(round(tfSampleb,1)); 
# Results4ShinyApp[7,2] = "deg. Celsius";
